import scipy, copy
"""
Collection of functions that compute best rigid transform ( rotation, translation, and scale ) between two point sets
"""
def icp_transform(box1,box2):
	epsilon=1e-6
	def find_closest_point(p,box):
		min_dist=1e30
		for i in range(len(box)):
			dist=(p[0]-box[i][0])* (p[0]-box[i][0])+(p[1]-box[i][1])* (p[1]-box[i][1])
			if dist<min_dist:
				min_dist=dist
				ret=i
		return [ret, scipy.sqrt(min_dist)]
	
	init_trans=scipy.mean(scipy.array(box1))-scipy.mean(scipy.array(box2))
	init_trans=[0,0]
	print "Init trans=",init_trans, 
	boxnew=apply_transform(box2,init_trans[0],init_trans[1],0.,1.)
	box_dist=[]
	for i in range(len(box1)): box_dist.append([i]+find_closest_point(box1[i], boxnew))
	box_dist.sort(lambda a, b: cmp(a[-1],b[-1]))
	last_p=0; cmean=box_dist[0][-1]
	for i in range(1,len(box_dist)):
		#if box_dist[i][-1]> 2 * cmean: break
		if box_dist[i][-1]> 1000: break
		cmean=((i+1)*cmean+box_dist[i][-1])/(i+2)
		last_p=i
	box11=[0]*(last_p+1)
	box22=[0]*(last_p+1)
	for i in range(last_p+1):
		box11[i]=box1[box_dist[i][0]]
		box22[i]=box2[box_dist[i][1]]
	#print "len 11=",len(box11), " Len 22=",len(box22), "Dist=", box_dist
	trans=find_transform(box11,box22)
	#print "trans=", trans
	error1=calculate_error(box11,box22,trans[0],trans[1],trans[2],trans[3])

	boxnew=apply_transform(box2,trans[0],trans[1],trans[2],trans[3])
	box_dist=[]
	for i in range(len(box1)): box_dist.append([i]+find_closest_point(box1[i], boxnew))
	box_dist.sort(lambda a, b: cmp(a[-1],b[-1]))
	last_p=0; cmean=box_dist[0][-1]
	for i in range(1,len(box_dist)):
		#if box_dist[i][-1]> 2 * cmean: break
		if box_dist[i][-1]> 1000: break
		cmean=((i+1)*cmean+box_dist[i][-1])/(i+2)
		last_p=i
	box11=[0]*(last_p+1)
	box22=[0]*(last_p+1)
	for i in range(last_p+1):
		box11[i]=box1[box_dist[i][0]]
		box22[i]=box2[box_dist[i][1]]
	trans=find_transform(box11,box22)
	error2=calculate_error(box11,box22,trans[0],trans[1],trans[2],trans[3])
	#print "trans=", trans
	print error1, error2,
	
	while scipy.mean(scipy.fabs(error2-error1))>epsilon:
		error1=error2
		boxnew=apply_transform(box2,trans[0],trans[1],trans[2],trans[3])
		box_dist=[]
		for i in range(len(box1)): box_dist.append([i]+find_closest_point(box1[i], boxnew))
		box_dist.sort(lambda a, b: cmp(a[-1],b[-1]))
		last_p=0; cmean=box_dist[0][-1]
		for i in range(1,len(box_dist)):
			#if box_dist[i][-1]> 2 * cmean: break
			if box_dist[i][-1]> 1000: break
			cmean=((i+1)*cmean+box_dist[i][-1])/(i+2)
			last_p=i
		box11=[0]*(last_p+1)
		box22=[0]*(last_p+1)
		for i in range(last_p+1):
			box11[i]=box1[box_dist[i][0]]
			box22[i]=box2[box_dist[i][1]]
		trans=find_transform(box11,box22)
		#print "After 3 len1 %d len3 %d: " % (len(box11), len(box22))
		error2=calculate_error(box11,box22,trans[0],trans[1],trans[2],trans[3])
		print error2,
	print
	return (trans, box11, box22)

def apply_transform(box,dx,dy,ang,scale,center=[0,0],trans_first=0):
	abox=scipy.array(box,'d')
	ang2=scipy.pi*ang/180.
	rotm=scipy.array([[scipy.cos(ang2),-scipy.sin(ang2)],[scipy.sin(ang2),scipy.cos(ang2)]])
	box2=copy.copy(box)
	if trans_first:
		for i in range(len(box)):
			box2[i]=scale*scipy.matrixmultiply(rotm,abox[i]-center+[dx,dy])+center
	else:
		for i in range(len(box)):
			box2[i]=scale*scipy.matrixmultiply(rotm,abox[i]-center)+[dx,dy]+center
	return box2

def transform_with_trans_rot_center(dx,dy,ang,scale, center=[0,0]):
	"""return equivalent transform in the order of translation, rotation around given center,
	   the input transform is rotation around origin, then translation
	"""
	ang2=scipy.pi*ang/180.
	rotm=scipy.array([[scipy.cos(ang2),-scipy.sin(ang2)],[scipy.sin(ang2),scipy.cos(ang2)]])
	trans=scipy.matrixmultiply(scipy.transpose(rotm),scipy.array([dx,dy])-scipy.array(center))/scale+scipy.array(center)	
	return [trans[0],trans[1],ang,scale]

def transform_with_center(dx,dy,ang,scale, center=[0,0]):
	ang2=scipy.pi*ang/180.
	rotm=scipy.array([[scipy.cos(ang2),-scipy.sin(ang2)],[scipy.sin(ang2),scipy.cos(ang2)]])
	cen2=scale*scipy.matrixmultiply(rotm,center)-center
	return [dx+cen2[0],dy+cen2[1],ang,scale]

def find_transform(box1, box2):
	abox1=scipy.array(box1,'d')
	abox2=scipy.array(box2,'d')
	box1_mean=scipy.sum(abox1)/len(abox1)
	box2_mean=scipy.sum(abox2)/len(abox2)
	abox1_2=abox1-box1_mean
	abox2_2=abox2-box2_mean
	
	h = scipy.array([[0.,0.],[0.,0.]])
	sbox1=0.0
	sbox2=0.0
	for i in range(len(box1)):
		h[0,0]=h[0,0]+abox2_2[i,0]*abox1_2[i,0]
		h[0,1]=h[0,1]+abox2_2[i,0]*abox1_2[i,1]
		h[1,0]=h[1,0]+abox2_2[i,1]*abox1_2[i,0]
		h[1,1]=h[1,1]+abox2_2[i,1]*abox1_2[i,1]
		sbox1=sbox1+abox1_2[i,0]*abox1_2[i,0]+abox1_2[i,1]*abox1_2[i,1]
		sbox2=sbox2+abox2_2[i,0]*abox2_2[i,0]+abox2_2[i,1]*abox2_2[i,1]
	[u,x,v]=scipy.svd(h)
	rotm=scipy.matrixmultiply(v,scipy.transpose(u))
	ang=scipy.arcsin(rotm[1,0])*180./scipy.pi
	scale=scipy.sqrt(sbox1/sbox2)
	trans=box1_mean-scale*scipy.matrixmultiply(rotm,box2_mean)
	return (trans[0], trans[1], ang, scale)

def test_transform(box):
	abox=scipy.array(box,'d')
	rg=scipy.random.Random()
	scale=rg.random()*2.
	ang=rg.random()*180-90.
	dx=rg.random()*scipy.max(scipy.max(box))
	dy=rg.random()*scipy.max(scipy.max(box))
	ang2=scipy.pi*ang/180.
	rotm=scipy.array([[scipy.cos(ang2),-scipy.sin(ang2)],[scipy.sin(ang2),scipy.cos(ang2)]])
	box2=copy.copy(box)
	for i in range(len(box1)):
		box2[i]=scale*scipy.matrixmultiply(rotm,abox[i])+[dx,dy]
	transform=find_transform(box2,box)
	print "\nInput  transform=",dx,dy,ang,scale
	print "Output transform=",transform[0],transform[1],transform[2],transform[3]

def calculate_error(box1, box2, dx, dy, ang, scale=1.0, center=[0,0], trans_first=0):
	from scipy import LinearAlgebra
	import scipy
	ang2=scipy.pi*ang/180.
	rotm=scipy.array([[scipy.cos(ang2),-scipy.sin(ang2)],[scipy.sin(ang2),scipy.cos(ang2)]])
	abox1=scipy.array(box1,'d')
	abox2=scipy.array(box2,'d')
	error=scipy.array([0.,0.])
	if trans_first:
		for i in range(len(box1)):
			error=error+scipy.fabs(abox1[i]-(scale*scipy.matrixmultiply(rotm,abox2[i]-center+[dx,dy])+center))
			#error=error+abox1[i]-(scale*scipy.matrixmultiply(rotm,abox2[i]-center)+[dx,dy]+center)
	else:
		for i in range(len(box1)):
			error=error+scipy.fabs(abox1[i]-(scale*scipy.matrixmultiply(rotm,abox2[i]-center)+[dx,dy]+center))
			#error=error+abox1[i]-(scale*scipy.matrixmultiply(rotm,abox2[i]-center)+[dx,dy]+center)
	return error/len(box1)	
