#!/usr/bin/env python

import sys;
from string import split, atof
if len(sys.argv)<4:
    print "Usage:",sys.argv[0],"<X-Ray data> <EM data> <output EM data> [low=<low res>] [high=<high res>] [scaleonly]"
    sys.exit(2)

low=None;	high=None;	scaleonly=0
for opts in sys.argv[4:]:
    if opts[0:4] == "low=":
        low=atof(opts[4:])
    elif opts[0:5] == "high=":
        high=atof(opts[5:])
    elif opts=="scaleonly":
        scaleonly=1
    else:
        print sys.argv[0]+": unknown option \""+opts+"\""
        sys.exit(2)
        
from Scientific.Functions.LeastSquares import leastSquaresFit
from Scientific.Functions.Interpolation import InterpolatingFunction
from Scientific.IO.TextFile import TextFile
import Numeric; N = Numeric

# EM data file
tem_s=[];	tem_fc=[]
for line in TextFile(sys.argv[2]):
	x,y=split(line)
	tem_s.append(atof(x))
	tem_fc.append(atof(y))

em_s=N.array(tem_s);	em_fc=N.array(tem_fc)
del tem_s, tem_fc

if not low:
    low=em_s[0]
elif low>1:low=1./low
    
if not high:
    high=em_s[-1]
elif high>1:
    high=1./high
    
if low>=high or low<em_s[0] or high>em_s[-1]:
    print sys.argv[0]+": incorrect frequency range: "+`low`+"->"+`high`+"("+`1./low`+"A->"+`1/high`+ "A)"
    sys.exit(1)
else:
    print "Fitting EM curve to X-Ray curve in frequency range: "+`low`+" -> "+`high`+" ("+`1./low`+"A -> "+`1/high`+ "A)"

em = InterpolatingFunction((em_s,), em_fc)

# xray data file
xray_fc=[]
for line in TextFile(sys.argv[1]):
	tx,ty=split(line)
	x=atof(tx);	y=atof(ty)
	if x>=low and x<=high:
		xray_fc.append([x,y])
	
# The mathematical model: xray = em * a + b
def em2xray(parameters, x):
    a = parameters[0]
    b = parameters[1]
    return a*em(x)+b
    
# The mathematical model: xray = em * a
def em2xray2(parameters, x):
    a = parameters[0]
    return a*em(x)

if scaleonly:
    fit = leastSquaresFit(em2xray2, (1,), xray_fc)
    print "Fitted parameters: a="+`fit[0][0]`
    print "Fit error:", fit[1]
    em_fc_fitted=em_fc*fit[0][0]
else:
    fit = leastSquaresFit(em2xray, (1,0), xray_fc)
    print "Fitted parameters: a="+`fit[0][0]`+"\tb="+`fit[0][1]`
    print "Fit error:", fit[1]
    em_fc_fitted=em_fc*fit[0][0]+fit[0][1]
    
outfile=TextFile(sys.argv[3],"w")
for i in range(len(em_s)):
	if em_fc_fitted[i] > 0:
    		outfile.write(`em_s[i]`+'\t'+`em_fc_fitted[i]`+'\n')
outfile.close()
