#!/usr/bin/env python
import os, sys, getopt, string, time
import EM 

## Usage message:
Usage = "Usage: " + sys.argv[0]+ " <project def file> [--ctf] [--wave=<n>] [-t|--ttest] [--reset=<angstrom>] [--waveinmask] [--ccmlinmask] [--ttestinmask=<0|1>] [-s n|--split=<n>] [-r|--restart] [-c n|--cpu=<n>] [-v|--verbose]\n"
cpu	= 1
split=1
restart=0
ctf=0
wavecycles=0
ttest=0
converg=0
reset=0
waveinmask=0
ccmlinmask=0
ttestinmask=0
verbose	= 0 

if len(sys.argv) < 2:
	try:
		import gui_virus
		gui_virus.main()		
	except:
		print Usage
		sys.exit(1)
	else:
		sys.exit(0)		
elif len(sys.argv) > 2:
	try:
		optlist, junks = getopt.getopt(sys.argv[2:], 'rvc:s:t', ['verbose','cpu=','restart','split','ctf','reset=','wave=','ttest','converg','ttestinmask=','waveinmask','ccmlinmask'])
	except getopt.GetoptError:
   		# print help information and exit:
		print Usage
		sys.exit(2)
	if len(junks):
		for o in junks:
			print "Error: option '"+o+"' is not supported"
		print Usage
		sys.exit(3)
	for o, a in optlist:
		if o in ("-v", "--verbose"):
			verbose = verbose + 1
		if o in ("-c", "--cpu"):
			cpu=string.atoi(a)
		if o in ("-r", "--restart"):
			restart=1
		if o in ("-s", "--split"):
			split=string.atoi(a)
		if o in ("--ctf",):
			ctf=1
		if o in ("--wave",):
			wavecycles=string.atoi(a)
			if wavecycles<=0: wavecycles=1
		if o in ("--reset",):
			reset=string.atof(a)
		if o in ("-t", "--ttest"):
			ttest=1
		if o in ("--waveinmask",):
			waveinmask=1
		if o in ("--ccmlinmask",):
			ccmlinmask=1
		if o in ("--ttestinmask",):
			converginmask=string.atoi(a)
			
############################################################
if not os.path.exists(sys.argv[1]):
	print "ERROR: cannot find project file \"%s\"" % ( sys.argv[1] )
	print Usage
	sys.exit(4)

# load the Project configuration, returned in object 'proj'
proj=EM.Project()
try:
	proj.ReadProjectFile(sys.argv[1])
except:
	print "Error in parsing project definition file '"+ sys.argv[1]+"', please make sure it is in the correct format"
	print Usage
	sys.exit(4)
############################################################

############################################################
if not os.environ.has_key("SAVR_NO_LOG_USAGE"): EM.SendUsage()
############################################################

############################################################
# setup some control parameters
if waveinmask:		proj.WaveInMask=waveinmask
if ccmlinmask:		proj.CCMLInMask=ccmlinmask
if ttestinmask:	proj.ConvergInMask=converginmask
if ctf:			proj.CTFCorrect=ctf
if wavecycles>0:	proj.WaveCycles=wavecycles
if cpu>1:		proj.CPU=cpu
if ttest:		proj.TTest=ttest
if verbose:		proj.Verbose=verbose
############################################################

proj.ProjectFile=os.path.abspath(sys.argv[1])

try:
	if ttest: 
		proj.PlotFourierShellCorrelation()
		sys.exit(5)
	
	starttime=time.time()

	startcycle=0
	if restart:	proj.Clean()	
	if reset: proj.Run(reset=reset)
	else:
		proj.Run()

	if split>1:
		newprojs=proj.Split(split)
		for i in range(len(newprojs)):
			filename=os.path.splitext(sys.argv[1])[0]+`i`+".prj"
			newprojs[i].SaveProjectFile(file=filename)
		for i in range(len(newprojs)):
			#print "#####################################################"	
			#print newprojs[i]
			if restart:	newprojs[i].Clean()
			newprojs[i].Run()

	endtime=time.time()

	if proj.Verbose:
		print "%s: start processing project \"%s\" at %s, finish processing at %s, the whole process has taken %d seconds (%.2f hours) \n" % \
			(sys.argv[0],sys.argv[1],time.ctime(starttime),time.ctime(endtime), endtime-starttime, (endtime-starttime)/3600)

except KeyboardInterrupt:
	pass
